from typing import Any, Dict, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String

def addDevice(
    deviceType: String,
    deviceName: String,
    deviceProps: Dict[String, Any],
    description: Optional[String] = ...,
) -> None: ...
def getDeviceHostname(deviceName: String) -> String: ...
def listDevices() -> BasicDataset: ...
def refreshBrowse(deviceName: String) -> None: ...
def removeDevice(deviceName: String) -> None: ...
def restart(deviceName: String) -> None: ...
def setDeviceEnabled(deviceName: String, enabled: bool) -> None: ...
def setDeviceHostname(deviceName: String, hostname: String) -> None: ...



from java.lang import Object

class Aggregate:
    def getDesc(self) -> None: ...
    def getId(self) -> None: ...
    def getName(self) -> None: ...

class AggregateInfo(Object, Aggregate):
    def getDesc(self) -> None: ...
    def getId(self) -> None: ...
    def getName(self) -> None: ...



from typing import Any, Dict, List, Optional, Union

from com.inductiveautomation.ignition.common.script.adapters import PyJsonObjectAdapter
from java.lang import String

def alterDock(
    dockId: String,
    config: Optional[Dict[String, Any]] = ...,
    sessionId: String = ...,
    pageId: String = ...,
) -> None: ...
def alterLogging(
    remoteLoggingEnabled: bool = ...,
    level: String = ...,
    remoteLoggingLevel: String = ...,
    sessionId: String = ...,
    pageId: String = ...,
) -> None: ...
def authenticationChallenge(
    sessionId: String = ...,
    pageId: String = ...,
    idp: String = ...,
    forceAuth: bool = ...,
    timeout: int = ...,
    payload: Optional[Dict[String, Any]] = ...,
    framing: String = ...,
) -> None: ...
def closeDock(id: String, sessionId: String = ..., pageId: String = ...) -> None: ...
def closePage(
    message: Optional[String] = ..., sessionId: String = ..., pageId: String = ...
) -> None: ...
def closePopup(id: String, sessionId: String = ..., pageId: String = ...) -> None: ...
def closeSession(message: Optional[String] = ..., sessionId: String = ...) -> None: ...
def download(
    filename: String,
    data: Any,
    contentType: Optional[String] = ...,
    sessionId: String = ...,
    pageId: String = ...,
) -> None: ...
def getProjectInfo() -> Dict[String, Any]: ...
def getSessionInfo(
    usernameFilter: Optional[String] = ..., projectFilter: Optional[String] = ...
) -> List[PyJsonObjectAdapter]: ...
def isAuthorized(
    isAllOf: bool, securityLevels: List[String], sessionId: String = ...
) -> bool: ...
def login(
    sessionId: String = ..., pageId: String = ..., forceAuth: bool = ...
) -> None: ...
def logout(
    sessionId: String = ..., pageId: String = ..., message: String = ...
) -> None: ...
def navigate(
    page: String,
    url: Optional[String] = ...,
    view: Optional[String] = ...,
    params: Optional[Dict[String, String]] = ...,
    sessionId: String = ...,
    pageId: String = ...,
    newTab: bool = ...,
) -> None: ...
def navigateBack(sessionId: String = ..., pageId: String = ...) -> None: ...
def navigateForward(sessionId: String = ..., pageId: String = ...) -> None: ...
def openDock(
    id: String,
    sessionId: String = ...,
    pageId: String = ...,
    params: Optional[Dict[String, String]] = ...,
) -> None: ...
def openPopup(
    id: String,
    view: String,
    params: Optional[Dict[String, Any]] = ...,
    title: String = ...,
    position: Optional[Dict[String, Union[int, String]]] = ...,
    showCloseIcon: bool = ...,
    draggable: bool = ...,
    resizable: bool = ...,
    modal: bool = ...,
    overlayDismiss: bool = ...,
    sessionId: String = ...,
    pageId: String = ...,
    viewPortBound: bool = ...,
) -> None: ...
def print(
    message: String,
    sessionId: String = ...,
    pageId: String = ...,
    destination: String = ...,
) -> None: ...
def refresh(
    sessionId: Optional[String] = ..., pageId: Optional[String] = ...
) -> None: ...
def sendMessage(
    messageType: String,
    payload: Dict[String, String],
    scope: String = ...,
    sessionId: String = ...,
    pageId: String = ...,
) -> None: ...
def setTheme(
    name: String, sessionId: Optional[String] = ..., pageId: Optional[String] = ...
) -> None: ...
def toggleDock(
    id: String,
    sessionId: String = ...,
    pageId: String = ...,
    params: Optional[Dict[String, String]] = ...,
) -> None: ...
def togglePopup(
    id: String,
    view: String,
    params: Optional[Dict[String, Any]],
    title: String = ...,
    position: Optional[Dict[String, Union[int, String]]] = ...,
    showCloseIcon: bool = ...,
    draggable: bool = ...,
    resizable: bool = ...,
    modal: bool = ...,
    overlayDismiss: bool = ...,
    sessionId: String = ...,
    pageId: String = ...,
    viewPortBound: bool = ...,
) -> None: ...
def vibrateDevice(duration: int, sessionId: Optional[String] = ...) -> None: ...



from typing import Any, Callable, Dict, List, Optional

from com.inductiveautomation.ignition.common.script.builtin.http import JythonHttpClient
from java.lang import String

def getExternalIpAddress() -> String: ...
def getHostName() -> String: ...
def getIpAddress() -> String: ...
def getRemoteServers(runningOnly: Optional[bool] = ...) -> List[String]: ...
def httpClient(
    timeout: int = ...,
    bypass_cert_validation: bool = ...,
    username: Optional[String] = ...,
    password: Optional[String] = ...,
    proxy: Optional[String] = ...,
    cookie_policy: String = ...,
    redirect_policy: String = ...,
    version: String = ...,
    customizer: Optional[Callable[..., Any]] = ...,
) -> JythonHttpClient: ...
def httpDelete(
    url: String,
    contentType: Optional[String] = ...,
    connectTimeout: int = ...,
    readTimeout: int = ...,
    username: Optional[String] = ...,
    password: Optional[String] = ...,
    headerValues: Optional[Dict[String, String]] = ...,
    bypassCertValidation: bool = ...,
) -> String: ...
def httpGet(
    url: String,
    connectTimeout: int = ...,
    readTimeout: int = ...,
    username: Optional[String] = ...,
    password: Optional[String] = ...,
    headerValues: Optional[Dict[String, String]] = ...,
    bypassCertValidation: Optional[bool] = ...,
    useCaches: bool = ...,
    throwOnError: bool = ...,
) -> String: ...
def httpPost(url: String, *args: Any, **kwargs: Any) -> String: ...
def httpPut(url: String, *args: Any, **kwargs: Any) -> String: ...
def openURL(url: String, useApplet: Optional[bool] = ...) -> None: ...
def sendEmail(
    smtp: Optional[String] = ...,
    fromAddr: String = ...,
    subject: Optional[String] = ...,
    body: Optional[String] = ...,
    html: bool = ...,
    to: Optional[List[String]] = ...,
    attachmentNames: Optional[List[object]] = ...,
    attachmentData: Optional[List[object]] = ...,
    timeout: int = ...,
    username: Optional[String] = ...,
    password: Optional[String] = ...,
    priority: String = ...,
    smtpProfile: Optional[String] = ...,
    cc: Optional[List[String]] = ...,
    bcc: Optional[List[String]] = ...,
    retries: int = ...,
    replyTo: Optional[List[String]] = ...,
) -> None: ...



from typing import Any, Optional, Union

from java.io import Closeable
from java.lang import AutoCloseable, Class, Object, String, Throwable
from org.apache.commons.lang3.builder import ToStringStyle
from org.slf4j import Logger

class LoggerEx(Object):
    DEFAULT_TO_STRING_STYLE: ToStringStyle
    def createSubLogger(self, arg: Union[Class, String]) -> LoggerEx: ...
    def debug(self, message: String, t: Optional[Throwable] = ...) -> None: ...
    def debugDuration(self, message: String) -> Closeable: ...
    def debugEvent(self, message: String, *args: Any) -> None: ...
    def debugf(self, message: String, *args: Any) -> None: ...
    def error(self, message: String, t: Optional[Throwable] = ...) -> None: ...
    def errorEvent(self, message: String, *args: Any) -> None: ...
    def errorf(self, message: String, *args: Any) -> None: ...
    def fatal(self, message: String, t: Optional[Throwable] = ...) -> None: ...
    def getIdentObject(self) -> Object: ...
    def getLoggerSLF4J(self) -> Logger: ...
    def getName(self) -> String: ...
    def getToStringStyle(self) -> ToStringStyle: ...
    def info(self, message: String, t: Optional[Throwable] = ...) -> None: ...
    def infoDuration(self, message: String) -> Closeable: ...
    def infoEvent(self, message: String, *args: Any) -> None: ...
    def infof(self, message: String, *args: Any) -> None: ...
    def isDebugEnabled(self) -> bool: ...
    def isIdentObjectEnabled(self) -> bool: ...
    def isInfoEnabled(self) -> bool: ...
    def isTraceEnabled(self) -> bool: ...
    def mdcClose(self) -> None: ...
    def mdcPut(self, key: String, value: String) -> None: ...
    def mdcPutCloseable(self, key: String, value: String) -> LoggerEx.MDCCloseable: ...
    def mdcRemove(self, key: String) -> None: ...
    def mdcSet(self) -> LoggerEx.MDCCloseable: ...
    @staticmethod
    def newBuilder() -> LoggerEx.Builder: ...
    def setIdentObject(self, identObj: Object) -> None: ...
    def setToStringStyle(self, toStringStyle: ToStringStyle) -> None: ...
    def trace(self, message: String, t: Optional[Throwable] = ...) -> None: ...
    def traceDuration(self, message: String) -> Closeable: ...
    def traceEvent(self, message: String, *args: Any) -> None: ...
    def tracef(self, message: String, *args: Any) -> None: ...
    def warn(self, message: String, t: Optional[Throwable] = ...) -> None: ...
    def warnEvent(self, message: String, *args: Any) -> None: ...
    def warnf(self, message: String, *args: Any) -> None: ...

    class Builder(Object):
        def build(self, *args: Any) -> LoggerEx: ...
        def eventSystem(self, systemId: str) -> LoggerEx.Builder: ...
        def identObject(self, identObj: Object) -> LoggerEx.Builder: ...
        def mdcContext(self, *args: Object) -> LoggerEx.Builder: ...
        def mutableIdentObject(self, identObj: Object) -> LoggerEx.Builder: ...

    class MDCCloseable(Object, AutoCloseable):
        def close(self) -> None: ...



from com.palantir.ptoss.cinch.core import DefaultBindableModel

class ScheduleModel(DefaultBindableModel):
    def deselectDayOfWeek(self, day) -> None: ...
    def getEveryHour(self) -> None: ...
    def getHour(self) -> None: ...
    def getMinute(self) -> None: ...
    def getSelectedDays(self) -> None: ...
    def isAlldays(self) -> None: ...
    def isDaySelected(self, day) -> None: ...
    def isEveryHourEnabled(self) -> None: ...
    def isEveryMinuteEnabled(self) -> None: ...
    def isHourEnabled(self) -> None: ...
    def isMinuteEnabled(self) -> None: ...
    def selectDayOfWeek(self, day) -> None: ...
    def setAllDays(self, allDays) -> None: ...
    def setEveryHour(self, everyHour) -> None: ...
    def setHour(self, hour) -> None: ...
    def setHourEnabled(self, hourEnabled) -> None: ...
    def setMinute(self, minute) -> None: ...
    def setMinuteEnabled(self, minuteEnabled) -> None: ...
    def setSelectedDays(self, selectedDays) -> None: ...
    def toCronString(self) -> None: ...



from typing import Optional

from com.inductiveautomation.ignition.common.opc import BrowseElementType
from com.inductiveautomation.ignition.common.sqltags.model.types import DataType
from com.inductiveautomation.ignition.common.tags.config.types import TagObjectType
from java.lang import Class, Object

class BrowseTag(Object):
    dataType: DataType
    name: str
    fullPath: str
    path: str
    type: TagObjectType
    valueSource: str
    def __init__(
        self,
        name: str,
        path: str,
        fullPath: str,
        type_: TagObjectType,
        valueSource: str,
        dataType: DataType,
    ) -> None: ...
    def getDataType(self): ...
    def getFullPath(self): ...
    def getPath(self): ...
    def getTagType(self): ...
    def getValueSource(self): ...
    def isDB(self): ...
    def isExpression(self): ...
    def isFolder(self): ...
    def isMemory(self): ...
    def isOPC(self): ...
    def isQuery(self): ...
    def isUDT(self): ...

class OPCBrowseTag(Object):
    def __init__(
        self,
        opcServer: Optional[str] = ...,
        type_: Optional[BrowseElementType] = ...,
        displayName: Optional[str] = ...,
        displayPath: Optional[str] = ...,
        dataType: Optional[Class] = ...,
        opcItemPath: Optional[str] = ...,
    ) -> None: ...
    def getDataType(self): ...
    def getDisplayName(self): ...
    def getDisplayPath(self): ...
    def getOpcItemPath(self): ...
    def getOpcServer(self): ...
    def getType(self): ...



from typing import Optional

from com.inductiveautomation.factorypmi.application.script.builtin import PrintUtilities
from java.awt import Component
from java.awt.image import BufferedImage
from java.lang import String

LANDSCAPE: int
PORTRAIT: int

def createImage(component: Component) -> BufferedImage: ...
def createPrintJob(component: Component) -> PrintUtilities.JythonPrintJob: ...
def printToImage(component: Component, filename: Optional[String] = ...) -> None: ...



from typing import Any, Dict, List, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String

def executeAndDistribute(
    path: String,
    project: String = ...,
    parameters: Optional[Dict[String, int]] = ...,
    action: Optional[String] = ...,
    actionSettings: Optional[Dict[String, Any]] = ...,
) -> None: ...
def executeReport(
    path: String,
    project: String = ...,
    parameters: Optional[Dict[String, int]] = ...,
    fileType: String = ...,
) -> Any: ...
def getReportNamesAsDataset(
    project: Optional[String] = ..., includeReportName: Optional[bool] = ...
) -> BasicDataset: ...
def getReportNamesAsList(project: Optional[String] = ...) -> List[String]: ...



from typing import Any, Iterator

from com.inductiveautomation.ignition.common.config import BasicPropertySet
from org.python.core import PyObject

class AlarmEvent:
    def __iter__(self) -> Iterator[Any]: ...
    def acknowledge(self, ackData: EventData) -> None: ...
    def active(self, activeData) -> None: ...
    def clear(self, clearData) -> None: ...
    def getAckData(self) -> None: ...
    def getActiveData(self) -> None: ...
    def getClearedData(self) -> None: ...
    def getDisplayPath(self) -> None: ...
    def getId(self) -> None: ...
    def getLabel(self) -> None: ...
    def getLastEventState(self) -> None: ...
    def getName(self) -> None: ...
    def getNotes(self) -> None: ...
    def getPriority(self) -> None: ...
    def getSource(self) -> None: ...
    def getState(self) -> None: ...
    def isAcked(self) -> None: ...
    def isCleared(self) -> None: ...
    def isShelved(self) -> None: ...
    def iterator(self) -> None: ...

class EventData(BasicPropertySet):
    def __init__(self, *args: Any) -> None: ...
    def getTimeStamp(self) -> long: ...

class PyAlarmEvent(AlarmEvent):
    def __iter__(self) -> Iterator[Any]: ...
    def acknowledge(self, ackData) -> None: ...
    def active(self, activeData) -> None: ...
    def clear(self, clearData) -> None: ...
    def contains(self, property) -> None: ...
    def get(self, property) -> None: ...
    def getAckData(self) -> None: ...
    def getActiveData(self) -> None: ...
    def getClearedData(self) -> None: ...
    def getDisplayPath(self) -> None: ...
    def getId(self) -> None: ...
    def getLabel(self) -> None: ...
    def getLastEventState(self) -> None: ...
    def getName(self) -> None: ...
    def getNotes(self) -> None: ...
    def getOrDefault(self, property) -> None: ...
    def getOrElse(self, property, defaultValue) -> None: ...
    def getPriority(self) -> None: ...
    def getSource(self) -> None: ...
    def getState(self) -> None: ...
    def isAcked(self) -> None: ...
    def isCleared(self) -> None: ...
    def isShelved(self) -> None: ...
    def set(self, property, value) -> None: ...
    def setGlobal(self, property, value) -> None: ...
    def sourceEvent(self) -> None: ...

class PyAlarmEventImpl(PyAlarmEvent, PyObject):
    def __init__(self, event) -> None: ...
    def contains(self, property) -> None: ...
    def get(self, property) -> None: ...
    def getOrDefault(self, property) -> None: ...
    def getOrElse(self, property, defaultValue) -> None: ...
    def set(self, property, value) -> None: ...
    def setGlobal(self, property, value) -> None: ...
    def sourceEvent(self) -> None: ...



from typing import Any, Dict, List, Optional, Tuple, Union

from com.inductiveautomation.ignition.common.alarming.evaluation import ShelvedPath
from com.inductiveautomation.ignition.common.alarming.query import AlarmQueryResultImpl
from java.lang import String
from java.util import Date

def acknowledge(
    alarmIds: List[String], notes: String, username: Optional[String] = ...
) -> None: ...
def cancel(alarmIds: List[String]) -> None: ...
def createRoster(name: String, description: String) -> None: ...
def getRosters() -> Dict[String, List[String]]: ...
def getShelvedPaths() -> List[ShelvedPath]: ...
def listPipelines(projectName: String = ...) -> List[String]: ...
def queryJournal(
    startDate: Optional[Date] = ...,
    endDate: Optional[Date] = ...,
    journalName: Optional[String] = ...,
    priority: Optional[List[Union[String, int]]] = ...,
    state: Optional[List[Union[String, int]]] = ...,
    path: Optional[List[String]] = ...,
    source: Optional[List[String]] = ...,
    displaypath: Optional[List[String]] = ...,
    all_properties: Optional[List[Tuple[String, String, Any]]] = ...,
    any_properties: Optional[List[Tuple[String, String, Any]]] = ...,
    defined: Optional[List[String]] = ...,
    includeData: Optional[bool] = ...,
    includeSystem: Optional[bool] = ...,
    isSystem: Optional[bool] = ...,
) -> AlarmQueryResultImpl: ...
def queryStatus(
    priority: Optional[List[Union[String, int]]] = ...,
    state: Optional[List[Union[String, int]]] = ...,
    path: Optional[List[String]] = ...,
    source: Optional[List[String]] = ...,
    displaypath: Optional[List[String]] = ...,
    all_properties: Optional[List[Tuple[String, String, Any]]] = ...,
    any_properties: Optional[List[Tuple[String, String, Any]]] = ...,
    defined: Optional[List[String]] = ...,
    includeShelved: bool = ...,
) -> AlarmQueryResultImpl: ...
def shelve(
    path: List[String], timeoutSeconds: int = ..., timeoutMinutes: int = ...
) -> None: ...
def unshelve(path: List[String]) -> None: ...



from typing import Any, List

from com.inductiveautomation.ignition.common.browsing import Results
from com.inductiveautomation.ignition.common.model.values import QualityCode
from com.inductiveautomation.ignition.common.sqltags.history import AggregateInfo
from java.lang import String
from java.util import Date

def browse(root: String) -> List[Results]: ...
def getAggregates(serverName: String) -> List[AggregateInfo]: ...
def getAttributes(serverName: String) -> List[AggregateInfo]: ...
def getServers() -> List[String]: ...
def insert(
    serverName: String, itemId: String, value: Any, date: Any, quality: int
) -> QualityCode: ...
def insertReplace(
    serverName: String, itemId: String, value: Any, date: Date, quality: int
) -> QualityCode: ...
def isServerAvailable(serverName: String) -> bool: ...
def readAttributes(
    serverName: String,
    itemId: String,
    attributeIds: String,
    startDate: Date,
    endDate: Date,
) -> List[Any]: ...
def readProcessed(
    serverName: String,
    itemIds: List[String],
    startDate: Date,
    endDate: Date,
    resampleIntervalMS: int,
    aggregates: List[Any],
) -> List[Any]: ...
def readRaw(
    serverName: String,
    itemIds: List[String],
    startDate: Date,
    endDate: Date,
    maxValues: int,
    boundingValues: bool,
) -> List[Any]: ...
def replace(
    serverName: String, itemId: String, value: Any, date: Date, quality: int
) -> QualityCode: ...



from typing import Any, List, Optional, Union

from com.inductiveautomation.ignition.common.model.values import (
    BasicQualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.opc import BasicOPCBrowseElement
from com.inductiveautomation.ignition.common.script.builtin import AbstractOPCUtilities
from com.inductiveautomation.ignition.common.script.builtin.ialabs import OPCBrowseTag
from java.lang import String

BrowseServerResult = List[Union[BasicOPCBrowseElement, AbstractOPCUtilities.PyOPCTag]]

def browse(
    opcServer: String, device: String, folderPath: String, opcItemPath: String
) -> List[OPCBrowseTag]: ...
def browseServer(opcServer: String, nodeId: String) -> BrowseServerResult: ...
def browseSimple(
    opcServer: String, device: String, folderPath: String, opcItemPath: String
) -> List[OPCBrowseTag]: ...
def getServerState(opcServer: String) -> Optional[String]: ...
def getServers(includeDisabled: Optional[bool] = ...) -> List[String]: ...
def isServerEnabled(serverName: String) -> bool: ...
def readValue(opcServer: String, itemPath: String) -> BasicQualifiedValue: ...
def readValues(
    opcServer: String, itemPaths: List[String]
) -> List[BasicQualifiedValue]: ...
def setServerEnabled(serverName: String, enabled: bool) -> None: ...
def writeValue(opcServer: String, itemPath: String, value: Any) -> QualityCode: ...
def writeValues(
    opcServer: String, itemPaths: List[String], values: List[Any]
) -> List[QualityCode]: ...



from typing import List

from com.inductiveautomation.ignition.common.user import User as User
from com.palantir.ptoss.cinch.core import DefaultBindableModel as DefaultBindableModel
from java.lang import String as String

class RosterModel(DefaultBindableModel):
    def getName(self) -> String: ...
    def getUsers(self) -> List[String]: ...
    def set(self, that: RosterModel) -> None: ...
    def setName(self, name: String) -> None: ...
    def setUsers(self, users: List[User]) -> None: ...



from typing import Any, List, Optional, Set, TypeVar

from java.io import Closeable, InputStream, OutputStream
from java.lang import Class, Enum, Object, String
from java.nio.channels import SocketChannel as SocketChannel

T = TypeVar("T")

class SocketAddress(Object):
    def __init__(self) -> None: ...

class FileNameMap:
    def getContentTypeFor(self, fileName: String) -> String: ...

class SocketImplFactory:
    def createSocketImpl(self) -> SocketImpl: ...

class SocketOption:
    def name(self) -> String: ...
    def type(self) -> Class: ...

class InetAddress(Object):
    def getAddress(self) -> bytearray: ...
    @staticmethod
    def getAllByName(host: String) -> List[InetAddress]: ...
    @staticmethod
    def getByAddress(*args: Any) -> InetAddress: ...
    @staticmethod
    def getByName(host: String) -> InetAddress: ...
    def getCanonicalHostName(self) -> String: ...
    def getHostAddress(self) -> String: ...
    def getHostName(self) -> String: ...
    @staticmethod
    def getLocalHost() -> InetAddress: ...
    @staticmethod
    def getLoopbackAddress() -> InetAddress: ...
    def isAnyLocalAddress(self) -> bool: ...
    def isLinkLocalAddress(self) -> bool: ...
    def isLoopbackAddress(self) -> bool: ...
    def isMCGlobal(self) -> bool: ...
    def isMCLinkLocal(self) -> bool: ...
    def isMCNodeLocal(self) -> bool: ...
    def isMCOrgLocal(self) -> bool: ...
    def isMCSiteLocal(self) -> bool: ...
    def isMulticastAddress(self) -> bool: ...
    def isReachable(self, *args: Any) -> bool: ...
    def isSiteLocalAddress(self) -> bool: ...

class InetSocketAddress(SocketAddress):
    def __init__(self, *args: Any) -> None: ...
    @staticmethod
    def createUnresolved(host: String, port: int) -> InetSocketAddress: ...
    def getAddress(self) -> InetAddress: ...
    def getHostname(self) -> String: ...
    def getHostString(self) -> String: ...
    def getPort(self) -> int: ...
    def isUnresolved(self) -> bool: ...

class Proxy(Object):
    NO_PROXY: Proxy
    def __init__(self, type_: Proxy.Type, sa: SocketAddress) -> None: ...
    def address(self) -> SocketAddress: ...
    def type(self) -> Proxy.Type: ...

    class Type(Enum):
        DIRECT: Proxy.Type
        HTTP: Proxy.Type
        SOCKS: Proxy.Type
        @staticmethod
        def values() -> List[Proxy.Type]: ...

class Socket(Object, Closeable):
    def __init__(self, *args: Any) -> None: ...
    def bind(self, bindpoint: SocketAddress) -> None: ...
    def close(self) -> None: ...
    def connect(
        self, endpoint: SocketAddress, timeout: Optional[int] = ...
    ) -> None: ...
    def getChannel(self) -> SocketChannel: ...
    def getInetAddress(self) -> InetAddress: ...
    def getInputStream(self) -> InputStream: ...
    def getKeepAlive(self) -> bool: ...
    def getLocalAddress(self) -> InetAddress: ...
    def getLocalPort(self) -> int: ...
    def getLocalSocketAddress(self) -> SocketAddress: ...
    def getOOBInline(self) -> bool: ...
    def getOption(self, name: SocketOption) -> T: ...
    def getOutputStream(self) -> OutputStream: ...
    def getPort(self) -> int: ...
    def getReceiveBufferSize(self) -> int: ...
    def getRemoteSocketAddress(self) -> SocketAddress: ...
    def getReuseAddress(self) -> bool: ...
    def getSendBufferSize(self) -> int: ...
    def getSoLinger(self) -> int: ...
    def getSoTimeout(self) -> int: ...
    def getTcpNoDelay(self) -> bool: ...
    def getTrafficClass(self) -> int: ...
    def isBound(self) -> bool: ...
    def isClosed(self) -> bool: ...
    def isConnected(self) -> bool: ...
    def isInputShutdown(self) -> bool: ...
    def isOutputShutdown(self) -> bool: ...
    def sendUrgentData(self, data: int) -> None: ...
    def setKeepAlive(self, on: bool) -> None: ...
    def setOOBInline(self, on: bool) -> None: ...
    def setOption(self, name: SocketOption, value: T) -> Socket: ...
    def setPerformancePreferences(
        self, connectionTime: int, latency: int, bandwidth: int
    ) -> None: ...
    def setReceiveBufferSize(self, size: int) -> None: ...
    def setReuseAddress(self, on: bool) -> None: ...
    def setSendBufferSize(self, size: int) -> None: ...
    @staticmethod
    def setSocketImplFactory(fac: SocketImplFactory) -> None: ...
    def setSoLinger(self, on: bool, linger: int) -> None: ...
    def setSoTimeout(self, timeout: int) -> None: ...
    def setTcpNoDelay(self, on: bool) -> None: ...
    def setTrafficClass(self, tc: int) -> None: ...
    def shutdownInput(self) -> None: ...
    def shutdownOutput(self) -> None: ...
    def supportedOptions(self) -> Set[SocketOption]: ...

class SocketImpl(Object):
    def __init__(self) -> None: ...



from typing import Any

from java.lang import Object

class SerialScriptModule(Object):
    def __init__(self) -> None: ...
    def setTrialExpired(self, arg) -> None: ...

    class PortManager:
        def __init__(self, *args: Any) -> None: ...
        def __enter__(self) -> None: ...
        def __exit__(self, exc_type, exc_val, exc_tb) -> None: ...

    class SerialConfigurator:
        def setBitRate(self, value) -> None: ...
        def setDataBits(self, value) -> None: ...
        def setFlowControl(self, value) -> None: ...
        def setHandshake(self, value) -> None: ...
        def setHardwareFlowControl(self, value) -> None: ...
        def setParity(self, value) -> None: ...
        def setStopBits(self, value) -> None: ...

    class SerialPortWrapper:
        def readBytes(self, port, numberOfBytes, timeout: int = ...) -> None: ...
        def readBytesAsString(
            self, port, numberOfBytes, timeout: int = ..., encoding: str = ...
        ) -> None: ...
        def readLine(
            self,
            port,
            timeout: int = ...,
            encoding: str = ...,
            crlfRequired: bool = ...,
        ) -> None: ...
        def readUntil(
            self, port, delimiter, includeDelimiter, timeout: int = ...
        ) -> None: ...
        def sendBreak(self, port, millis) -> None: ...
        def write(self, port, toWrite, encoding: str = ...) -> None: ...
        def writeBytes(self, port, toWrite, timeout: int = ...) -> None: ...



from typing import List

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String

def getAccounts() -> List[String]: ...
def getAccountsDataset() -> BasicDataset: ...
def getPhoneNumbers(accountName: String) -> List[String]: ...
def getPhoneNumbersDataset(accountName: String) -> BasicDataset: ...
def sendSms(
    accountName: String, fromNumber: String, toNumber: String, message: String
) -> None: ...



from java.lang import Object

class JythonHttpClient(Object):
    def delete(self, *args, **kwargs) -> None: ...
    def deleteAsync(self, *args, **kwargs) -> None: ...
    def get(self, *args, **kwargs) -> None: ...
    def getAsync(self, *args, **kwargs) -> None: ...
    def getConnectTimeout(self) -> None: ...
    def getCookieManager(self) -> None: ...
    def getJavaClient(self) -> None: ...
    def getRedirectPolicy(self) -> None: ...
    def head(self, *args, **kwargs) -> None: ...
    def headAsync(self, *args, **kwargs) -> None: ...
    def options(self, *args, **kwargs) -> None: ...
    def parseChart(self, contentType) -> None: ...
    def patch(self, *args, **kwargs) -> None: ...
    def patchAsync(self, *args, **kwargs) -> None: ...
    def post(self, *args, **kwargs) -> None: ...
    def postAsync(self, *args, **kwargs) -> None: ...
    def put(self, *args, **kwargs) -> None: ...
    def putAsync(self, *args, **kwargs) -> None: ...
    def request(self, *args, **kwargs) -> None: ...
    def requestAsync(self, *args, **kwargs) -> None: ...
    def setGson(self, gson) -> None: ...
    def trace(self, *args, **kwargs) -> None: ...
    def traceAsync(self, *args, **kwargs) -> None: ...



from typing import Any, Optional

from java.lang import Object, String
from java.util import EventObject

class AWTEvent(EventObject):
    def __init__(self, source: Object, id: int) -> None: ...
    def getID(self) -> int: ...
    def paramString(self) -> String: ...
    def setSource(self, newSource: Object) -> None: ...

class Color(Object):
    def __init__(self, *args: Any) -> None: ...

class Component(Object):
    def imageUpdate(
        self, img: Image, infoflags: int, x: int, y: int, width: int, height: int
    ) -> bool: ...

class Container(Component):
    def add(self, *args: Any) -> Optional[Component]: ...

class Image(Object):
    def flush(self) -> None: ...
    def getAccelerationPriority(self) -> float: ...
    def setAccelerationPriority(self, priority: float) -> None: ...

class Toolkit(Object):
    def __init__(self) -> None: ...
    def beep(self) -> None: ...
    @staticmethod
    def getDefaultToolkit() -> Toolkit: ...

class Window(Container):
    def __init__(self, *args: Any) -> None: ...

class Frame(Window):
    def __init__(self, *args: Any) -> None: ...

class Graphics(Object):
    def create(self) -> Graphics: ...



from typing import Any, Dict, List, Optional, Type, Union

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.script.builtin import DatasetUtilities
from java.lang import String
from java.util import Date, Locale

ColType = Union[Date, float, int, str, unicode]

def addColumn(
    dataset: BasicDataset,
    colIndex: int,
    col: List[Any],
    colName: String,
    colType: Type[ColType],
) -> BasicDataset: ...
def addRow(dataset: BasicDataset, rowIndex: int, row: List[Any]) -> BasicDataset: ...
def addRows(
    dataset: BasicDataset, rowIndex: int, rows: List[List[Any]]
) -> BasicDataset: ...
def appendDataset(dataset1: BasicDataset, dataset2: BasicDataset) -> BasicDataset: ...
def clearDataset(dataset: BasicDataset) -> BasicDataset: ...
def dataSetToHTML(
    showHeaders: bool, dataset: BasicDataset, title: String
) -> String: ...
def deleteRow(dataset: BasicDataset, rowIndex: int) -> BasicDataset: ...
def deleteRows(dataset: BasicDataset, rowIndices: List[int]) -> BasicDataset: ...
def exportCSV(filename: String, showHeaders: bool, dataset: BasicDataset) -> String: ...
def exportExcel(
    filename: String,
    showHeaders: bool,
    dataset: List[BasicDataset],
    nullsEmpty: bool = ...,
) -> String: ...
def exportHTML(
    filename: String, showHeaders: bool, dataset: BasicDataset, title: String
) -> String: ...
def filterColumns(
    dataset: BasicDataset, columns: Union[List[String], List[int]]
) -> BasicDataset: ...
def formatDates(
    dataset: BasicDataset, dateFormat: String, locale: Optional[Locale] = ...
) -> BasicDataset: ...
def fromCSV(csv: String) -> BasicDataset: ...
def getColumnHeaders(dataset: BasicDataset) -> List[String]: ...
def setValue(
    dataset: BasicDataset, rowIndex: int, columnName: String, value: Any
) -> BasicDataset: ...
def sort(
    dataset: BasicDataset, keyColumn: Union[String, int], ascending: bool = ...
) -> BasicDataset: ...
def toCSV(
    dataset: BasicDataset,
    showHeaders: bool = ...,
    forExport: bool = ...,
    localized: bool = ...,
) -> String: ...
def toDataSet(*args: Any) -> BasicDataset: ...
def toExcel(
    showHeaders: bool,
    dataset: List[BasicDataset],
    nullsEmpty: bool = ...,
    sheetNames: Optional[List[String]] = ...,
) -> Any: ...
def toPyDataSet(dataset: Dataset) -> DatasetUtilities.PyDataSet: ...
def updateRow(
    dataset: BasicDataset, rowIndex: int, changes: Dict[String, Any]
) -> BasicDataset: ...



from typing import Any, Dict, Optional, Union

from com.inductiveautomation.factorypmi.application import FPMIWindow
from java.awt import Component
from java.awt.image import BufferedImage
from java.lang import Object, String
from java.util import EventObject
from org.python.core import PyObject

class INavUtilities:
    def centerWindow(self, arg: Union[FPMIWindow, String]) -> None: ...
    def closeParentWindow(self, event: EventObject) -> None: ...
    def closeWindow(self, arg: Union[FPMIWindow, String]) -> None: ...
    def getCurrentWindow(self) -> String: ...
    def goBack(self) -> PyObject: ...
    def goForward(self) -> PyObject: ...
    def goHome(self) -> PyObject: ...
    def openWindow(
        self, path: String, params: Optional[Dict[String, Any]] = ...
    ) -> PyObject: ...
    def openWindowImpl(
        self, path: String, params: Dict[String, Any], openAdditional: bool
    ) -> PyObject: ...
    def openWindowInstance(
        self, path: String, params: Optional[Dict[String, Any]] = ...
    ) -> PyObject: ...
    def swapTo(self, name: String, params: Dict[String, Any]) -> PyObject: ...
    def swapWindow(self, *args: Any) -> PyObject: ...

class NavUtilities(INavUtilities):
    def centerWindow(self, arg: Union[FPMIWindow, String]) -> None: ...
    def closeParentWindow(self, event: EventObject) -> None: ...
    def closeWindow(self, arg: Union[FPMIWindow, String]) -> None: ...
    def getCurrentWindow(self) -> String: ...
    def goBack(self) -> PyObject: ...
    def goForward(self) -> PyObject: ...
    def goHome(self) -> PyObject: ...
    def openWindow(
        self, path: String, params: Optional[Dict[String, Any]] = ...
    ) -> PyObject: ...
    def openWindowImpl(
        self, path: String, params: Dict[String, Any], openAdditional: bool
    ) -> PyObject: ...
    def openWindowInstance(
        self, path: String, params: Optional[Dict[String, Any]] = ...
    ) -> PyObject: ...
    def swapTo(self, name: String, params: Dict[String, Any]) -> PyObject: ...
    def swapWindow(self, *args: Any) -> PyObject: ...

class PrintUtilities(Object):
    def __init__(self, app: Any) -> None: ...
    def createImage(self, c: Component) -> BufferedImage: ...
    def createPrintJob(self, c: Component) -> PrintUtilities.JythonPrintJob: ...
    def printToImage(self, c: Component, fileName: Optional[str] = ...) -> None: ...

    class JythonPrintJob(Object):
        def getBottomMargin(self) -> None: ...
        def getLeftMargin(self) -> None: ...
        def getOrientation(self) -> None: ...
        def getPageHeight(self) -> None: ...
        def getPageWidth(self) -> None: ...
        def getPrinterName(self) -> None: ...
        def getRightMargin(self) -> None: ...
        def getTopMargin(self) -> None: ...
        def getZoomFactor(self) -> None: ...
        def isFitToPage(self) -> None: ...
        def isShowPrintDialog(self) -> None: ...
        def setBottomMargin(self, bottomMargin) -> None: ...
        def setFitToPage(self, fitToPage) -> None: ...
        def setLeftMargin(self, leftMargin) -> None: ...
        def setMargins(self, m) -> None: ...
        def setOrientation(self, orientation) -> None: ...
        def setPageHeight(self, pageHeight) -> None: ...
        def setPageWidth(self, pageWidth) -> None: ...
        def setPrinterName(self, printerName) -> None: ...
        def setRightMargin(self, rightMargin) -> None: ...
        def setShowPrintDialog(self, showPrintDialog) -> None: ...
        def setZoomFactor(self, zoomFactor) -> None: ...

class WindowUtilities(Object):
    def confirm(self, *args) -> None: ...
    def errorBox(self, *args) -> None: ...
    def inputBox(self, *args) -> None: ...
    def messageBox(self, *args) -> None: ...
    def passwordBox(self, *args) -> None: ...
    def warningBox(self, *args) -> None: ...



from typing import Union

from com.inductiveautomation.ignition.common import Path
from com.inductiveautomation.ignition.common.config import Property
from java.lang import String

class TagPath(Path):
    def compareTo(self, that: TagPath) -> int: ...
    def getChildPath(self, arg: Union[Property, String]) -> TagPath: ...
    def getItemName(self) -> String: ...
    def getLastPathComponent(self) -> String: ...
    def getParentPath(self) -> Path: ...
    def getPathComponent(self, i: int) -> String: ...
    def getPathLength(self) -> int: ...
    def getProperty(self) -> Property: ...
    def getSource(self) -> String: ...
    def isAncestorOf(self, path: Path) -> bool: ...
    def toStringFull(self) -> String: ...
    def toStringPartial(self) -> String: ...



from typing import Any, List, Optional, Sequence

from com.inductiveautomation.ignition.common.model.values import QualityCode
from java.lang import Object, String

class Result:
    def getDisplayPath(self) -> None: ...
    def getPath(self) -> None: ...
    def getType(self) -> None: ...
    def hasChildren(self) -> None: ...

class Results(Object):
    continuationPoint: Optional[String]
    resultQuality: QualityCode
    results: Sequence[Any]
    totalAvailableResults: int
    def __init__(self, *args: Any) -> None: ...
    @staticmethod
    def error(result: Results) -> Results: ...
    def getContinuationPoint(self) -> Optional[String]: ...
    def getResultQuality(self) -> QualityCode: ...
    def getResults(self) -> Sequence[Any]: ...
    def getReturnedSize(self) -> int: ...
    def getTotalAvailableSize(self) -> int: ...
    @staticmethod
    def of(results: List[Any]) -> Results: ...
    def setContinuationPoint(
        self, continuationPoint: Optional[String] = ...
    ) -> None: ...
    def setResultQuality(self, value: QualityCode) -> None: ...
    def setResults(self, results: List[Any]) -> None: ...
    def setTotalAvailableResults(self, totalAvailableResults: int) -> None: ...



from typing import TypeVar

from java.lang import AutoCloseable, Runnable
from java.util import Iterator, Spliterator
from java.util.function import Consumer

S = TypeVar("S")
T = TypeVar("T")

class BaseStream(AutoCloseable):
    def close(self) -> None: ...
    def isParallel(self) -> bool: ...
    def iterator(self) -> Iterator: ...
    def onClose(self, closeHandler: Runnable) -> S: ...
    def parallel(self) -> S: ...
    def sequential(self) -> S: ...
    def spliterator(self) -> Spliterator: ...
    def unordered(self) -> S: ...

class Stream(BaseStream):
    @staticmethod
    def builder() -> Stream.Builder: ...
    def close(self) -> None: ...
    def isParallel(self) -> bool: ...
    def iterator(self) -> Iterator: ...
    def onClose(self, closeHandler: Runnable) -> S: ...
    def parallel(self) -> S: ...
    def sequential(self) -> S: ...
    def spliterator(self) -> Spliterator: ...
    def unordered(self) -> S: ...

    class Builder(Consumer):
        def accept(self, t: T) -> None: ...
        def add(self, t: T) -> Stream.Builder: ...
        def build(self) -> None: ...



from typing import Any, Dict, List, Optional, Set, TypeVar, Union

from java.lang import Object, String
from java.time import Instant, ZoneId
from java.util.function import (
    Consumer,
    Function,
    ToDoubleFunction,
    ToIntFunction,
    ToLongFunction,
)

E = TypeVar("E")
T = TypeVar("T")

class Comparator:
    def compare(self, o1: T, o2: T) -> int: ...
    @staticmethod
    def comparing(keyExtractor: Function, keyComparator: Comparator) -> Comparator: ...
    @staticmethod
    def comparingDouble(keyExtractor: ToDoubleFunction) -> Comparator: ...
    @staticmethod
    def comparingInt(keyExtractor: ToIntFunction) -> Comparator: ...
    @staticmethod
    def comparingLong(keyExtractor: ToLongFunction) -> Comparator: ...
    def equals(self, obj: Object) -> bool: ...
    @staticmethod
    def naturalOrder() -> Comparator: ...
    @staticmethod
    def nullsFirst(comparator: Comparator) -> Comparator: ...
    @staticmethod
    def nullsLast(comparator: Comparator) -> Comparator: ...
    def reversed(self) -> Comparator: ...
    @staticmethod
    def reverseOrder() -> Comparator: ...
    def thenComparing(self, *args: Any) -> Comparator: ...
    def thenComparingDouble(self, keyExtractor: ToDoubleFunction) -> Comparator: ...
    def thenComparingInt(self, keyExtractor: ToIntFunction) -> Comparator: ...
    def thenComparingLong(self, keyExtractor: ToLongFunction) -> Comparator: ...

class Iterator:
    def forEachRemaining(self, action: Consumer) -> None: ...
    def hasNext(self) -> bool: ...
    def next(self) -> E: ...
    def remove(self) -> bool: ...

class Spliterator:
    CONCURRENT: int
    DISTINCT: int
    IMMUTABLE: int
    NONNULL: int
    ORDERED: int
    SIZED: int
    SORTED: int
    SUBSIZED: int
    def characteristics(self) -> int: ...
    def estimateSize(self) -> long: ...
    def forEachRemaining(self, action: Consumer) -> None: ...
    def getComparator(self) -> Comparator: ...
    def getExactSizeIfKnown(self) -> long: ...
    def hasCharacteristics(self, characteristics: int) -> bool: ...
    def tryAdvance(self, action: Consumer) -> bool: ...
    def trySplit(self) -> Spliterator: ...

class Calendar(Object):
    ALL_STYLES: int
    AM: int
    AM_PM: int
    APRIL: int
    AUGUST: int
    DATE: int
    DAY_OF_MONTH: int
    DAY_OF_WEEK: int
    DAY_OF_WEEK_IN_MONTH: int
    DAY_OF_YEAR: int
    DECEMBER: int
    DST_OFFSET: int
    ERA: int
    FEBRUARY: int
    FIELD_COUNT: int
    FRIDAY: int
    HOUR: int
    HOUR_OF_DAY: int
    JANUARY: int
    JULY: int
    JUNE: int
    LONG: int
    LONG_FORMAT: int
    LONG_STANDALONE: int
    MARCH: int
    MAY: int
    MILLISECOND: int
    MINUTE: int
    MONDAY: int
    MONTH: int
    NARROW_FORMAT: int
    NARROW_STANDALONE: int
    NOVEMBER: int
    OCTOBER: int
    PM: int
    SATURDAY: int
    SECOND: int
    SEPTEMBER: int
    SHORT: int
    SHORT_FORMAT: int
    SHORT_STANDALONE: int
    SUNDAY: int
    THURSDAY: int
    TUESDAY: int
    UNDECIMBER: int
    WEDNESDAY: int
    WEEK_OF_MONTH: int
    WEEK_OF_YEAR: int
    YEAR: int
    ZONE_OFFSET: int
    def add(self, field: int, amount: int) -> None: ...
    def after(self, when: Object) -> bool: ...
    def before(self, when: Object) -> bool: ...
    def clear(self, field: Optional[int] = ...) -> None: ...
    def clone(self) -> Object: ...
    def compareTo(self, anotherCalendar: Calendar) -> int: ...
    def get(self, field: int) -> int: ...
    def getActualMaximum(self, field: int) -> int: ...
    def getActualMinimum(self, field: int) -> int: ...
    @staticmethod
    def getAvailableCalendarTypes() -> Set[String]: ...
    @staticmethod
    def getAvailableLocales() -> List[Locale]: ...
    def getCalendarType(self) -> String: ...
    def getDisplayName(self, field: int, style: int, locale: Locale) -> String: ...
    def getDisplayNames(
        self, field: int, style: int, locale: Locale
    ) -> Dict[String, int]: ...
    def getFirstDayOfWeek(self) -> int: ...
    def getGreatestMinimum(self, field: int) -> int: ...
    def getInstance(self, *args: Any) -> Calendar: ...
    def getTimeZone(self) -> TimeZone: ...
    def getWeeksInWeekYear(self) -> int: ...
    def getWeekYear(self) -> int: ...
    def isLenient(self) -> bool: ...
    def isSet(self, field: int) -> bool: ...
    def isWeekDateSUpported(self) -> bool: ...
    def roll(self, field: int, amount: int) -> None: ...
    def set(self, *args: int) -> None: ...
    def setFirstDayOfWeek(self, value: int) -> None: ...
    def setLenient(self, lenient: bool) -> None: ...
    def setMinimalDaysInFirstWeek(self, value: int) -> None: ...
    def setTime(self, date: Date) -> None: ...
    def setTimeInMillis(self, millis: long) -> None: ...
    def setTimeZone(self, value: TimeZone) -> None: ...
    def setWeekDate(self, weekYear: int, weekOfYear: int, dayOfWeek: int) -> None: ...
    def toInstant(self) -> Instant: ...

class Currency(Object):
    @staticmethod
    def getAvailableCurrencies() -> Set[Currency]: ...
    def getCurrencyCode(self) -> String: ...
    def getDisplayName(self, locale: Optional[Locale] = ...) -> String: ...
    @staticmethod
    def getInstance(arg: Union[Locale, String]) -> Currency: ...
    def getNumericCode(self) -> int: ...
    def getNumericCodeAsString(self) -> String: ...
    def getSymbol(self, locale: Optional[Locale] = ...) -> String: ...

class Date(Object):
    def __init__(self, date: Optional[long] = ...) -> None: ...
    def after(self, when: Date) -> bool: ...
    def before(self, when: Date) -> bool: ...
    def compareTo(self, anotherDate: Date) -> int: ...
    def getTime(self) -> long: ...
    def setTime(self, time: long) -> None: ...

class EventObject(Object):
    def __init__(self, source: Object) -> None: ...
    def getSource(self) -> Object: ...

class Locale(Object):
    country: Optional[str]
    language: str
    variant: Optional[str]
    def __init__(
        self, language: str, country: Optional[str] = ..., variant: Optional[str] = ...
    ) -> None: ...
    def CANADA(self) -> Locale: ...
    def CANADA_FRENCH(self) -> Locale: ...
    def CHINA(self) -> Locale: ...
    def CHINESE(self) -> Locale: ...
    def ENGLISH(self) -> Locale: ...
    def FRANCE(self) -> Locale: ...
    def FRENCH(self) -> Locale: ...
    def GERMAN(self) -> Locale: ...
    def GERMANY(self) -> Locale: ...
    def ITALIAN(self) -> Locale: ...
    def ITALY(self) -> Locale: ...
    def JAPAN(self) -> Locale: ...
    def JAPANESE(self) -> Locale: ...
    def KOREA(self) -> Locale: ...
    def KOREAN(self) -> Locale: ...
    def PRC(self) -> Locale: ...
    def SIMPLIFIED_CHINESE(self) -> Locale: ...
    def TAIWAN(self) -> Locale: ...
    def TRADITIONAL_CHINESE(self) -> Locale: ...
    def UK(self) -> Locale: ...
    def US(self) -> Locale: ...

class TimeZone(Object):
    LONG: int
    SHORT: int
    def clone(self) -> Object: ...
    @staticmethod
    def getAvailableIDs(rawOffset: Optional[int] = ...) -> List[String]: ...
    @staticmethod
    def getDefault() -> TimeZone: ...
    def getDisplayName(self, *args: Any) -> String: ...
    def getDSTSavings(self) -> int: ...
    def getID(self) -> String: ...
    def getOffset(self, *args: Any) -> int: ...
    def getRawOffset(self) -> int: ...
    @staticmethod
    def getTimeZone(arg: Union[String, ZoneId]) -> TimeZone: ...
    def hasSameRules(self, other: TimeZone) -> bool: ...
    def isDaylightTime(self, date: Date) -> bool: ...
    def observesDaylightTime(self) -> bool: ...
    @staticmethod
    def setDefault(zone: TimeZone) -> None: ...
    def setID(self, ID: String) -> None: ...
    def setRawOffset(self, offsetMillis: int) -> None: ...
    def toZoneId(self) -> ZoneId: ...
    def useDaylightTime(self) -> bool: ...

class UUID(Object):
    def __init__(self, mostSigBits: long, leastSigBits: long) -> None: ...
    def clockSequence(self) -> int: ...
    def compareTo(self, val: UUID) -> int: ...
    @staticmethod
    def fromString(name: String) -> UUID: ...
    def getLeastSignificantBits(self) -> long: ...
    def getMostSignificantBits(self) -> long: ...
    @staticmethod
    def nameUUIDFromBytes(name: bytearray) -> UUID: ...
    def node(self) -> long: ...
    @staticmethod
    def randomUUID() -> UUID: ...
    def timestamp(self) -> long: ...
    def variant(self) -> int: ...
    def version(self) -> int: ...



from typing import List, Optional

from com.inductiveautomation.ignition.common.i18n import LocalizedString
from com.inductiveautomation.ignition.common.model.values import Quality, QualityCode
from java.lang import Enum, Object, String
from java.util import Locale

class DataQuality(Object):
    @staticmethod
    def fromIntValue(value: int) -> DataQuality: ...
    def getDescription(self) -> LocalizedString: ...
    def getIntValue(self) -> int: ...
    def getLevel(self) -> Quality.Level: ...
    def getName(self) -> str: ...
    def getQualityCode(self) -> QualityCode: ...
    @staticmethod
    def getQualityFor(value: int) -> DataQuality: ...
    def isDataUsed(self) -> bool: ...
    def isGood(self) -> bool: ...
    def isGoodData(self) -> bool: ...
    def isOpcBadData(self) -> bool: ...
    def toString(self, locale: Optional[Locale] = ...) -> String: ...
    @staticmethod
    def valueOf(name: str) -> DataQuality: ...
    @staticmethod
    def values() -> List[DataQuality]: ...
    @staticmethod
    def worstOf(q1: DataQuality, q2: DataQuality) -> DataQuality: ...
    @staticmethod
    def worstOfAll(*args: DataQuality) -> DataQuality: ...

class DataType(Enum):
    def fromIntValue(self, val) -> None: ...
    def getComponentDataType(self) -> None: ...
    def getIntValue(self) -> None: ...
    def getJavaType(self) -> None: ...
    def getTypeClass(self) -> None: ...
    def getTypeForClass(self, clazz) -> None: ...
    def getTypeForValue(self, val) -> None: ...
    def isArray(self) -> None: ...
    def isFloatingPoint(self) -> None: ...
    def isNumeric(self) -> None: ...
    def legacyDataType(self) -> None: ...
    def values(self) -> None: ...



from typing import Any, Dict, List, Optional, Set, Union

from java.lang import Number, Object, String, StringBuffer
from java.math import RoundingMode
from java.util import Calendar, Currency, Date, Locale, TimeZone

class CharacterIterator:
    DONE: str
    def clone(self) -> str: ...
    def current(self) -> str: ...
    def first(self) -> str: ...
    def getBeginIndex(self) -> int: ...
    def getEndIndex(self) -> int: ...
    def getIndex(self) -> int: ...
    def last(self) -> str: ...
    def next(self) -> str: ...
    def previous(self) -> str: ...
    def setIndex(self, position: int) -> str: ...

class AttributedCharacterIterator(CharacterIterator):
    def clone(self) -> str: ...
    def current(self) -> str: ...
    def first(self) -> str: ...
    def getAllAttributeKeys(self) -> Set[AttributedCharacterIterator.Attribute]: ...
    def getAttribute(
        self, attribute: AttributedCharacterIterator.Attribute
    ) -> Object: ...
    def getAttributes(self) -> Dict[AttributedCharacterIterator.Attribute, Object]: ...
    def getBeginIndex(self) -> int: ...
    def getEndIndex(self) -> int: ...
    def getIndex(self) -> int: ...
    def last(self) -> str: ...
    def next(self) -> str: ...
    def previous(self) -> str: ...
    def setIndex(self, position: int) -> str: ...

    class Attribute(Object):
        INPUT_METHOD_SEGMENT: AttributedCharacterIterator.Attribute
        LANGUAGE: AttributedCharacterIterator.Attribute
        READING: AttributedCharacterIterator.Attribute

class FieldPosition(Object):
    def __init__(self, *args: Any) -> None: ...
    def getBeginIndex(self) -> int: ...
    def getEndIndex(self) -> int: ...
    def getField(self) -> Any: ...
    def getFieldAttribute(self) -> Format.Field: ...
    def setBeginIndex(self, bi: int) -> None: ...
    def setEndIndex(self, ei: int) -> None: ...

class Format(Object):
    def clone(self) -> Object: ...
    def formatToCharacterIterator(self, obj: Object) -> AttributedCharacterIterator: ...
    def parseObject(
        self, source: String, pos: Optional[ParsePosition] = ...
    ) -> Object: ...

    class Field(AttributedCharacterIterator.Attribute): ...

class DateFormat(Format):
    AM_PM_FIELD: int
    DATE_FIELD: int
    DAY_OF_WEEK_FIELD: int
    DAY_OF_WEEK_IN_MONTH_FIELD: int
    DAY_OF_YEAR_FIELD: int
    DEFAULT: int
    ERA_FIELD: int
    FULL: int
    HOUR_OF_DAY0_FIELD: int
    HOUR_OF_DAY1_FIELD: int
    HOUR0_FIELD: int
    HOUR1_FIELD: int
    LONG: int
    MEDIUM: int
    MILLISECOND_FIELD: int
    MONTH_FIELD: int
    SECOND_FIELD: int
    SHORT: int
    TIMEZONE_FIELD: int
    WEEK_OF_MONTH_FIELD: int
    WEEK_OF_YEAR_FIELD: int
    YEAR_FIELD: int
    def format(
        self,
        date: Union[Date, Object],
        toAppendTo: Optional[StringBuffer] = ...,
        pos: Optional[FieldPosition] = ...,
    ) -> Union[String, StringBuffer]: ...
    @staticmethod
    def getAvailableLocales() -> List[Locale]: ...
    def getCalendar(self) -> Calendar: ...
    @staticmethod
    def getDateInstance(
        style: Optional[int] = ..., aLocale: Optional[Locale] = ...
    ) -> DateFormat: ...
    @staticmethod
    def getDateTimeInstance() -> DateFormat: ...
    @staticmethod
    def getInstance() -> DateFormat: ...
    def getNumberFormat(self) -> NumberFormat: ...
    @staticmethod
    def getTimeInstance(style: int, aLocale: Optional[Locale] = ...) -> DateFormat: ...
    def getTimeZone(self) -> TimeZone: ...
    def isLenient(self) -> bool: ...
    def parse(self, source: String) -> Date: ...
    def setCalendar(self, newCalendar: Calendar) -> None: ...
    def setLenient(self, lenient: bool) -> None: ...
    def setNumberFormat(self, newNumberFormat: NumberFormat) -> None: ...
    def setTimeZone(self, zone: TimeZone) -> None: ...

class DateFormatSymbols(Object):
    def __init__(self, locale: Optional[Locale] = ...) -> None: ...
    def clone(self) -> Object: ...
    def getAmPmStrings(self) -> List[String]: ...
    @staticmethod
    def getAvailableLocales() -> List[Locale]: ...
    def getEras(self) -> List[String]: ...
    @staticmethod
    def getInstance(locale: Optional[Locale] = ...) -> DateFormatSymbols: ...
    def getLocalPatternChars(self) -> String: ...
    def getMonths(self) -> List[String]: ...
    def getShortMonths(self) -> List[String]: ...
    def getShortWeekdays(self) -> List[String]: ...
    def getWeekdays(self) -> List[String]: ...
    def getZoneStrings(self) -> List[List[String]]: ...
    def setAmPmStrings(self, newAmpms: List[String]) -> None: ...
    def setEras(self, newEras: List[String]) -> None: ...
    def setLocalPatternChars(self, newLocalPatternChars: String) -> None: ...
    def setMonths(self, newMonths: List[String]) -> None: ...
    def setShortMonths(self, newShortMonths: List[String]) -> None: ...
    def setShortWeekdays(self, newShortWeekdays: List[String]) -> None: ...
    def setWeekdays(self, newWeekdays: List[String]) -> None: ...
    def setZoneStrings(self, newZoneStrings: List[List[String]]) -> None: ...

class NumberFormat(Format):
    FRACTION_FIELD: int
    INTEGER_FIELD: int
    def format(
        self,
        number: Union[float, long, Object],
        toAppendTo: Optional[StringBuffer] = ...,
        pos: Optional[FieldPosition] = ...,
    ) -> Union[String, StringBuffer]: ...
    @staticmethod
    def getAvailableLocales() -> List[Locale]: ...
    def getCurrency(self) -> Currency: ...
    @staticmethod
    def getCurrencyInstance(inLocale: Optional[Locale] = ...) -> NumberFormat: ...
    @staticmethod
    def getInstance(inLocale: Optional[Locale] = ...) -> NumberFormat: ...
    @staticmethod
    def getIntegerInstance(inLocale: Optional[Locale] = ...) -> NumberFormat: ...
    def getMaximumFractionDigits(self) -> int: ...
    def getMaximumIntegerDigits(self) -> int: ...
    def getMinimumFractionDigits(self) -> int: ...
    def getMinimumIntegerDigits(self) -> int: ...
    @staticmethod
    def getNumberInstance(inLocale: Optional[Locale] = ...) -> NumberFormat: ...
    @staticmethod
    def getPercentInstance(inLocale: Optional[Locale] = ...) -> NumberFormat: ...
    def getRoundingMode(self) -> RoundingMode: ...
    def isGroupingUsed(self) -> bool: ...
    def isParseIntegerOnly(self) -> bool: ...
    def parse(self, source: String) -> Number: ...
    def setCurrency(self, currency: Currency) -> None: ...
    def setGroupingUsed(self, newValue: bool) -> None: ...
    def setMaximumFractionDigits(self, newValue: int) -> None: ...
    def setMaximumIntegerDigits(self, newValue: int) -> None: ...
    def setMinimumFractionDigits(self, newValue: int) -> None: ...
    def setMinimumIntegerDigits(self, newValue: int) -> None: ...
    def setParseIntegerOnly(self, value: bool) -> None: ...
    def setRoundingMode(self, roundingMode: RoundingMode) -> None: ...

    class Field(Format.Field):
        CURRENCY: NumberFormat.Field
        DECIMAL_SEPARATOR: NumberFormat.Field
        EXPONENT: NumberFormat.Field
        EXPONENT_SIGN: NumberFormat.Field
        EXPONENT_SYMBOL: NumberFormat.Field
        FRACTION: NumberFormat.Field
        GROUPING_SEPARATOR: NumberFormat.Field
        INTEGER: NumberFormat.Field
        PERCENT: NumberFormat.Field
        PERMILLE: NumberFormat.Field
        SIGN: NumberFormat.Field

class ParsePosition(Object):
    def __init__(self, index: int) -> None: ...
    def getErrorIndex(self) -> int: ...
    def getIndex(self) -> int: ...
    def setErrorIndex(self, ei: int) -> None: ...
    def setIndex(self, index: int) -> None: ...

class SimpleDateFormat(DateFormat):
    def __init__(
        self,
        pattern: Optional[String] = ...,
        arg: Union[DateFormatSymbols, Locale, None] = ...,
    ) -> None: ...
    def applyLocalizedPattern(self, pattern: String) -> None: ...
    def applyPattern(self, pattern: String) -> None: ...
    def get2DigitYearStart(self) -> Date: ...
    def getDateFormatSymbols(self) -> DateFormatSymbols: ...
    def set2DigitYearStart(self, startDate: Date) -> None: ...
    def setDateFormatSymbols(self, newFormatSymbols: DateFormatSymbols) -> None: ...
    def toLocalizedPattern(self) -> String: ...
    def toPattern(self) -> String: ...



class ClientContext:
    def addPropertyChangeListener(self, *args) -> None: ...
    def desearilize(self, data, log) -> None: ...
    def getAuthProfileName(self) -> None: ...
    def getDefaultDataSourceName(self) -> None: ...
    def getDefaultTagProviderName(self) -> None: ...
    def getExecutionManager(self) -> None: ...
    def getLaunchContext(self) -> None: ...
    def getLocalizationManager(self) -> None: ...
    def getModules(self) -> None: ...
    def getNamedQueryManager(self) -> None: ...
    def getProgressManager(self) -> None: ...
    def getProject(self) -> None: ...
    def getProjectName(self) -> None: ...
    def getRootPaneContainer(self) -> None: ...
    def getTagManager(self) -> None: ...
    def getTagPollRate(self) -> None: ...
    def getUIEventBus(self) -> None: ...
    def removePropertyChangeListened(self, *args) -> None: ...



from typing import TypeVar

from java.lang import Object

R = TypeVar("R")
T = TypeVar("T")
U = TypeVar("U")

class BiPredicate:
    def negate(self) -> BiPredicate: ...
    def test(self, t: T, u: U) -> bool: ...

class Consumer:
    def accept(self, t: T) -> None: ...
    def andThen(self, after: Consumer) -> Consumer: ...

class Function:
    def andThen(self, after: Function) -> Function: ...
    def apply(self, t: T) -> R: ...
    def compose(self, before: Function) -> Function: ...
    @staticmethod
    def identity() -> Function: ...

class Predicate:
    @staticmethod
    def isEqual(targetRef: Object) -> Predicate: ...
    def negate(self) -> Predicate: ...
    def test(self, t: T) -> bool: ...

class ToDoubleFunction:
    def applyAsDouble(self, value: T) -> object: ...

class ToIntFunction:
    def applyAsInt(self, value: T) -> object: ...

class ToLongFunction:
    def applyAsLong(self, value: T) -> object: ...



from typing import Any, Dict, List, Optional, Tuple

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String

def copyEquipment(
    equipmentSource: String,
    newEquipmentName: String,
    enabled: bool,
    activeAddress: String,
    activePort: int,
    passiveAddress: String,
    passivePort: int,
    deviceId: int,
    dbTablePrefix: Optional[String] = ...,
    description: Optional[String] = ...,
) -> None: ...
def deleteToolProgram(ppid: String) -> None: ...
def enableDisableEquipment(
    enable: bool, names: Tuple[String, ...]
) -> List[unicode]: ...
def getResponse(
    transactionID: int,
    equipment: String,
    timeout: Optional[int] = ...,
    poll: Optional[int] = ...,
) -> Any: ...
def getToolProgram(ppid: String) -> Dict[String, Any]: ...
def getToolProgramDataset() -> BasicDataset: ...
def sendRequest(
    streamFunction: String, reply: bool, body: Any, equipment: String
) -> int: ...
def sendResponse(
    transactionID: int,
    systemBytes: int,
    streamFunction: String,
    body: Any,
    equipment: String,
) -> None: ...
def startSimEventRun(simulatorName: String, eventRunName: String) -> None: ...
def toDataSet(secsObject: Any) -> BasicDataset: ...
def toTreeDataSet(dataset: BasicDataset) -> BasicDataset: ...



from typing import Optional

from com.inductiveautomation.ignition.common.gson import Gson as Gson
from com.inductiveautomation.ignition.common.gson import JsonObject as JsonObject
from com.inductiveautomation.ignition.common.jsonschema import JsonSchema as JsonSchema
from com.inductiveautomation.ignition.common.util import LoggerEx as LoggerEx
from java.lang import Object
from java.lang import String as String
from java.util.function import Consumer

class PerspectiveModule(Object):
    LOG_PREFIX: String
    META_SCHEMA: JsonSchema
    MODULE_ID: String
    SESSION_PROPS_SCHEMA: JsonSchema
    VIEW_SCHEMA: JsonSchema
    @staticmethod
    def createPerspectiveCompatibleGson(
        customize: Optional[Consumer] = ...,
    ) -> Gson: ...
    @staticmethod
    def defaultMetaProps() -> JsonObject: ...
    @staticmethod
    def defaultSessionProps() -> JsonObject: ...
    @staticmethod
    def defaultViewProps() -> JsonObject: ...
    @staticmethod
    def getLogger(name: String) -> LoggerEx: ...



from typing import Any, List

from java.lang import Enum, Object, Thread
from java.time.temporal import ChronoUnit

class TimeUnit(Enum):
    DAYS: TimeUnit
    HOURS: TimeUnit
    MICROSECONDS: TimeUnit
    MILLISECONDS: TimeUnit
    MINUTES: TimeUnit
    NANOSECONDS: TimeUnit
    SECONDS: TimeUnit
    def convert(self, *args: Any) -> long: ...
    @staticmethod
    def of(chronoUnit: ChronoUnit) -> TimeUnit: ...
    def sleep(self, timeout: long) -> None: ...
    def timedJoin(self, thread: Thread, timeout: long) -> None: ...
    def timedWait(self, obj: Object, timeout: long) -> None: ...
    def toChronoUnit(self) -> ChronoUnit: ...
    def toDays(self, duration: long) -> long: ...
    def toHours(self, duration: long) -> long: ...
    def toMicros(self, duration: long) -> long: ...
    def toMillis(self, duration: long) -> long: ...
    def toMinutes(self, duration: long) -> long: ...
    def toNanos(self, duration: long) -> long: ...
    def toSeconds(self, duration: long) -> long: ...
    @staticmethod
    def values() -> List[TimeUnit]: ...



from typing import Dict, List

from com.inductiveautomation.ignition.common.user import PyUser
from java.lang import String

def addUsers(rosterName: String, users: List[PyUser]) -> None: ...
def createRoster(name: String, description: String) -> None: ...
def deleteRoster(rosterName: String) -> None: ...
def getRoster(name): ...
def getRosterNames() -> List[String]: ...
def getRosters() -> Dict[String, List[String]]: ...
def removeUsers(rosterName: String, users: List[PyUser]) -> None: ...



from typing import Any, Optional, Tuple, Union

from java.io import InputStream
from java.lang import Object, String

class ApplicationScope(Object):
    ALL: int
    CLIENT: int
    DESIGNER: int
    GATEWAY: int
    NONE: int
    @staticmethod
    def getGlobalScope() -> int: ...
    @staticmethod
    def getScopePrefix() -> String: ...
    @staticmethod
    def init(globalScope: int) -> None: ...
    @staticmethod
    def isClient(scope: int) -> bool: ...
    @staticmethod
    def isDesigner(scope: int) -> bool: ...
    @staticmethod
    def isGateway(scope: int) -> bool: ...
    @staticmethod
    def parseScope(s: String) -> int: ...
    @staticmethod
    def toCode(scope: int) -> String: ...

class Version(Object):
    build: int
    dev: bool
    major: int
    minor: int
    rev: int
    beta: int
    rc: int
    snapshot: bool
    def __init__(
        self,
        major: int = ...,
        minor: int = ...,
        rev: int = ...,
        build: int = ...,
        beta: int = ...,
        rc: int = ...,
    ) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def compareTo(self, that: Version) -> Optional[int]: ...
    def exists(self) -> bool: ...
    @staticmethod
    def fromXML(inputStream: InputStream) -> Version: ...
    def getBasicString(self) -> str: ...
    def getBeta(self) -> int: ...
    def getBuildNumber(self) -> int: ...
    def getMajor(self) -> int: ...
    def getMinor(self) -> int: ...
    def getRc(self) -> int: ...
    def getRevision(self) -> int: ...
    def getXML(self) -> str: ...
    def isDev(self) -> bool: ...
    def isFutureVersion(self, arg: Union[Version, String]) -> bool: ...
    def isSnapshot(self) -> bool: ...
    @staticmethod
    def parse(s: String) -> Version: ...
    def toParseableString(self) -> str: ...
    def toString(self) -> str: ...
    def toTuple(self, strict: bool = ...) -> Tuple[int, ...]: ...



class JythonMap:
    def __contains__(self, item) -> None: ...
    def __finditem__(self, key) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> None: ...
    def get(self, *args) -> None: ...
    def has_key(self, pyKey) -> None: ...
    def items(self) -> None: ...
    def iteritems(self) -> None: ...
    def iterkeys(self) -> None: ...
    def itervalues(self) -> None: ...
    def keys(self) -> None: ...
    def values(self) -> None: ...

class JythonSequence:
    def __contains__(self, item) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> None: ...
    def __mul__(self, other) -> None: ...
    def __rmul__(self, other) -> None: ...
    def count(self, element) -> None: ...
    def index(self, element) -> None: ...

class MutableJythonMap:
    def __delitem__(self, key) -> None: ...
    def __setitem__(self, key, value) -> None: ...
    def clear(self) -> None: ...
    def pop(self, *args) -> None: ...
    def popitem(self) -> None: ...
    def setdefault(self, *args) -> None: ...
    def update(self, *args, **kwargs) -> None: ...

class MutableJythonSequence:
    def __add__(self, other) -> None: ...
    def __imul__(self, other) -> None: ...
    def append(self, element) -> None: ...
    def extend(self, sequence) -> None: ...
    def insert(self, index, element) -> None: ...
    def pop(self, *args) -> None: ...
    def remove(self, element) -> None: ...
    def sort(self, *args, **kwargs) -> None: ...

class AbstractJythonMap(JythonMap):
    def __contains__(self, item) -> None: ...
    def __finditem__(self, key) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> None: ...
    def get(self, *args) -> None: ...
    def has_key(self, pyKey) -> None: ...
    def items(self) -> None: ...
    def iteritems(self) -> None: ...
    def iterkeys(self) -> None: ...
    def itervalues(self) -> None: ...
    def keys(self) -> None: ...
    def values(self) -> None: ...

class AbstractJythonSequence(JythonSequence):
    def __contains__(self, item) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> None: ...
    def __mul__(self, other) -> None: ...
    def __rmul__(self, other) -> None: ...
    def count(self, element) -> None: ...
    def index(self, element) -> None: ...

class AbstractMutableJythonMap(MutableJythonMap):
    def __delitem__(self, key) -> None: ...
    def __setitem__(self, key, value) -> None: ...
    def clear(self) -> None: ...
    def pop(self, *args) -> None: ...
    def popitem(self) -> None: ...
    def setdefault(self, *args) -> None: ...
    def update(self, *args, **kwargs) -> None: ...

class AbstractMutableJythonSequence(MutableJythonSequence):
    def __add__(self, other) -> None: ...
    def __imul__(self, other) -> None: ...
    def append(self, element) -> None: ...
    def extend(self, sequence) -> None: ...
    def insert(self, index, element) -> None: ...
    def pop(self, *args) -> None: ...
    def remove(self, element) -> None: ...
    def sort(self, *args, **kwargs) -> None: ...



from typing import Any, Dict, List, Optional, Tuple, Union

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.model.values import QualityCode
from com.inductiveautomation.ignition.common.opc import BrowseElementType
from com.inductiveautomation.ignition.common.script.abc import AbstractJythonSequence
from com.inductiveautomation.ignition.common.script.message import Request
from java.lang import Class
from java.lang import Exception as JavaException
from java.lang import Object, String
from org.python.core import PyFunction, PyObject
from org.slf4j import Logger

class AbstractOPCUtilities(Object):
    def browseServer(
        self, opcServer: String, nodeId: String
    ) -> List[AbstractOPCUtilities.PyOPCTag]: ...
    def getServers(self) -> None: ...
    def getServerState(self, opcServer) -> None: ...
    def isServerEnabled(self, serverName) -> None: ...
    def readValue(self, opcServer, itemPath) -> None: ...
    def readValues(self, opcServer, itemPaths) -> None: ...
    def setServerEnabled(self, serverName, enabled) -> None: ...
    def writeValue(self, *args, **kwargs) -> None: ...
    def writeValues(self, *args, **kwargs) -> None: ...

    class PyOPCTag(PyObject):
        displayName: String
        elementType: BrowseElementType
        nodeId: String
        serverName: String
        def __init__(
            self,
            serverName: String,
            nodeId: String,
            displayName: String,
            elementType: BrowseElementType,
        ) -> None: ...
        def __findattr_ex__(self, name: String) -> PyObject: ...
        def getDisplayName(self) -> String: ...
        def getElementType(self) -> BrowseElementType: ...
        def getNodeId(self) -> String: ...
        def getServerName(self) -> String: ...

class DatasetUtilities(Object):
    @staticmethod
    def addColumn(*args) -> None: ...
    @staticmethod
    def addRow(*args) -> None: ...
    @staticmethod
    def addRows(*args) -> None: ...
    @staticmethod
    def appendDataset(ds1, ds2) -> None: ...
    @staticmethod
    def clearDataset(ds) -> None: ...
    @staticmethod
    def dataSetToExcel(headerRow, datasets) -> None: ...
    @staticmethod
    def dataSetToExcelBytes(headerRow, objects, nullsEmpty, sheetNames) -> None: ...
    @staticmethod
    def dataSetToExcelStreaming(headerRow, objects, out, nullsEmpty) -> None: ...
    @staticmethod
    def dataSetToHTML(headerRow, ds, title) -> None: ...
    @staticmethod
    def dataSetToHTMLStreaming(headerRow, ds, title, fw) -> None: ...
    @staticmethod
    def deleteRow(ds, row) -> None: ...
    @staticmethod
    def deleteRows(ds, rows) -> None: ...
    @staticmethod
    def filterColumns(dataset, columns) -> None: ...
    @staticmethod
    def formatDates(dataset, format, locale=...) -> None: ...
    @staticmethod
    def fromCSV(csv) -> None: ...
    @staticmethod
    def fromCSVJava(csv) -> None: ...
    @staticmethod
    def getColumnHeaders(ds) -> None: ...
    @staticmethod
    def insertColumn(*args) -> None: ...
    @staticmethod
    def insertRow(*args) -> None: ...
    @staticmethod
    def setValue(*args) -> None: ...
    @staticmethod
    def sort(
        ds: BasicDataset,
        keyColumn: Union[int, String],
        ascending: Optional[bool] = ...,
        naturalOrdering: Optional[bool] = ...,
    ) -> BasicDataset: ...
    @staticmethod
    def toCSV(*args, **kwargs) -> None: ...
    @staticmethod
    def toCSVJava(ds, showHeaders, forExport, localized: bool = ...) -> None: ...
    @staticmethod
    def toCSVJavaStreaming(ds, showHeaders, forExport, sw, localized) -> None: ...
    @staticmethod
    def toDataSet(*args) -> None: ...
    @staticmethod
    def toExcel(*args, **kwargs) -> None: ...
    @staticmethod
    def toJSONObject(data) -> None: ...
    @staticmethod
    def toPyDataSet(dataset: Dataset) -> PyDataSet: ...
    @staticmethod
    def updateRow(ds, row, changes) -> None: ...

    class PyDataSet(Dataset, AbstractJythonSequence):
        def __init__(self, ds: Optional[Dataset] = ...) -> None: ...
        def getColumnCount(self) -> int: ...
        def getColumnIndex(self, colName: String) -> int: ...
        def getColumnName(self, col: int) -> String: ...
        def getColumnNames(self) -> List[String]: ...
        def getColumnType(self, col: int) -> Class: ...
        def getColumnTypes(self) -> List[Class]: ...
        def getPrimitiveValueAt(self, row: int, col: int) -> float: ...
        def getQualityAt(self, row: int, col: int) -> QualityCode: ...
        def getRowCount(self) -> int: ...
        def getValueAt(self, row: int, col: Union[int, String]) -> Any: ...
        def setData(self, data: Dataset) -> None: ...

class SProcCall(Object):
    callFinished: bool
    datasource: String
    params: Dict[SProcCall.SProcArgKey, SProcCall.SProcArg]
    procedureName: String
    resultset: Dataset
    returnParam: SProcCall.SProcArg
    skipAudit: bool
    txId: String
    updateCount: int
    def getDataSource(self) -> String: ...
    def getOutParamValue(self, param: Union[int, String]) -> Any: ...
    def getProcedureName(self) -> String: ...
    def getResultSet(self) -> BasicDataset: ...
    def getReturnValue(self) -> Any: ...
    def getTxId(self) -> String: ...
    def getUpdateCount(self) -> int: ...
    def isSkipAudit(self) -> bool: ...
    def registerInParam(
        self, param: Union[int, String], typeCode: int, value: Any
    ) -> None: ...
    def registerOutParam(self, param: Union[int, String], typeCode: int) -> None: ...
    def registerReturnParam(self, typeCode: int) -> None: ...
    def setSkipAudit(self, skipAudit: bool) -> None: ...
    def setTxId(self, txId: String) -> None: ...

    class SProcArg(Object):
        outParam: bool
        inParam: bool
        paramType: int
        value: Object
        def getParamType(self) -> int: ...
        def getValue(self) -> Any: ...
        def isInParam(self) -> bool: ...
        def isOutParam(self) -> bool: ...
        def setParamType(self, paramType: int) -> None: ...
        def setValue(self, value: Object) -> None: ...

    class SProcArgKey(Object):
        index: int
        name: String
        def getParamIndex(self) -> int: ...
        def getParamName(self) -> String: ...
        def isNamedParam(self) -> bool: ...

class SystemUtilities(Object):
    @staticmethod
    def logger(loggerName: String) -> Logger: ...
    @staticmethod
    def parseTranslateArguments(
        *args: PyObject, **kwargs: String
    ) -> Tuple[String, String, bool]: ...

    class RequestImpl(Object, Request):
        timeout: int
        def __init__(self, timeout: int) -> None: ...
        def cancel(self) -> None: ...
        def checkTimeout(self) -> None: ...
        def finishExceptionally(self, e: JavaException) -> None: ...
        def finishSuccessfully(self, value: Object) -> None: ...
        def get(self) -> Object: ...
        def getError(self) -> JavaException: ...
        def getLongId(self) -> long: ...
        def onError(self, func: PyFunction) -> None: ...
        def onSuccess(self, func: PyFunction) -> None: ...



from typing import Any, List, Optional, Union

from com.inductiveautomation.ignition.common.config import Property
from com.inductiveautomation.ignition.common.tags.model import TagPath
from java.lang import Object, String

class AbstractTagPath(Object, TagPath):
    @staticmethod
    def compareNullLow(c1: Property, c2: Property) -> int: ...

class BasicTagPath(AbstractTagPath):
    def __init__(
        self,
        source: String,
        pathParts: Optional[List[String]] = ...,
        prop: Optional[Property] = ...,
    ) -> None: ...
    @staticmethod
    def append(root: TagPath, arg: Union[String, TagPath]) -> TagPath: ...
    @staticmethod
    def copy(path: TagPath) -> BasicTagPath: ...
    @staticmethod
    def renameParentFolder(path: TagPath, newParent: TagPath) -> BasicTagPath: ...
    @staticmethod
    def subPath(path: TagPath, *args: Any) -> BasicTagPath: ...

class PropertyAlteredTagPath(AbstractTagPath):
    def __init__(self, path: TagPath, prop: Property) -> None: ...

class SourceAlteredTagPath(AbstractTagPath):
    def __init__(self, path: TagPath, source: String) -> None: ...



from typing import Any, List, Optional

from java.lang import String

def fileExists(filepath: String) -> bool: ...
def getTempFile(extension: String) -> String: ...
def openFile(
    extension: Optional[String] = ..., defaultLocation: Optional[String] = ...
) -> Optional[String]: ...
def openFiles(
    extension: Optional[String] = ..., defaultLocation: Optional[String] = ...
) -> Optional[List[String]]: ...
def readFileAsBytes(filepath: String) -> Any: ...
def readFileAsString(filepath: String, encoding: Optional[String] = ...) -> String: ...
def saveFile(
    filename: String,
    extension: Optional[String] = ...,
    typeDesc: Optional[String] = ...,
) -> Optional[String]: ...
def writeFile(
    filepath: String, data: Any, append: bool = ..., encoding: String = ...
) -> None: ...



from java.lang import Object

class Annotation(Object):
    def delete(self) -> None: ...
    @staticmethod
    def fromJson(json) -> None: ...
    def getData(self) -> None: ...
    def getPath(self) -> None: ...
    def getRangeEnd(self) -> None: ...
    def getRangeStart(self) -> None: ...
    def getStorageId(self) -> None: ...
    def getType(self) -> None: ...
    def isDeleted(self) -> None: ...
    @staticmethod
    def newBuilder() -> None: ...
    @staticmethod
    def newDelete(storageId) -> None: ...
    def toJson(self) -> None: ...
    def updatePath(self, path) -> None: ...



from typing import Any, Dict, List, Tuple

from java.lang import String

def addConnection(
    name: String,
    description: String,
    discoveryUrl: String,
    endpointUrl: String,
    securityPolicy: String,
    securityMode: String,
    settings: Dict[String, Any],
) -> None: ...
def callMethod(
    connectionName: String, objectId: String, methodId: String, inputs: List[String]
) -> Tuple[Any, Any, Any]: ...
def removeConnection(name: String) -> bool: ...



from typing import Any, List, Optional, Set, TypeVar, Union

from com.inductiveautomation.ignition.common.document import DocumentElement
from com.inductiveautomation.ignition.common.gson import Gson, JsonElement
from com.inductiveautomation.ignition.common.model.values import (
    QualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.sqltags.model.types import (
    DataQuality,
    DataType,
)
from java.awt import Color
from java.lang import Class, Number, Object, String
from java.util import UUID, Comparator, Date, Locale
from org.json import JSONObject
from org.python.core import PyObject

T = TypeVar("T")

class Dataset:
    def binarySearch(self, column: int, key: Any) -> int: ...
    def getColumnAsList(self, col: int) -> List[Any]: ...
    def getColumnCount(self) -> int: ...
    def getColumnIndex(self, colName: String) -> int: ...
    def getColumnName(self, col: int) -> String: ...
    def getColumnNames(self) -> List[String]: ...
    def getColumnType(self, col: int) -> Class: ...
    def getColumnTypes(self) -> List[Class]: ...
    def getPrimitiveValueAt(self, row: int, col: int) -> float: ...
    def getQualityAt(self, row: int, col: int) -> QualityCode: ...
    def getRowCount(self) -> int: ...
    def getValueAt(self, row: int, col: Union[int, String]) -> Any: ...
    def hasQualityData(self) -> bool: ...

class AbstractDataset(Dataset):
    def __init__(
        self,
        columnNames: List[String],
        columnTypes: List[Class],
        qualityCodes: Optional[List[List[QualityCode]]] = ...,
    ) -> None: ...
    @staticmethod
    def convertToQualityCodes(
        dataQualities: List[List[DataQuality]],
    ) -> List[List[QualityCode]]: ...
    def getBulkQualityCodes(self) -> List[List[QualityCode]]: ...
    def getColumnCount(self) -> int: ...
    def getColumnIndex(self, colName: String) -> int: ...
    def getColumnName(self, col: int) -> String: ...
    def getColumnNames(self) -> List[String]: ...
    def getColumnType(self, col: int) -> Class: ...
    def getColumnTypes(self) -> List[Class]: ...
    def getPrimitiveValueAt(self, row: int, col: int) -> float: ...
    def getQualityAt(self, row: int, col: int) -> QualityCode: ...
    def getRowCount(self) -> int: ...
    def getValueAt(self, row: int, col: Union[int, String]) -> Any: ...
    def setColumnNames(self, arg: List[str]) -> None: ...
    def setColumnTypes(self, arg: List[Class]) -> None: ...
    def setDirty(self) -> None: ...

class BasicDataset(AbstractDataset):
    def __init__(self, *args: Any) -> None: ...
    def columnContainsNulls(self, col: int) -> bool: ...
    def datasetContainsNulls(self) -> bool: ...
    def getData(self) -> Any: ...
    def setAllDirectly(
        self, columnNames: List[String], columnTypes: List[Class], data: Any
    ) -> None: ...
    def setDataDirectly(self, arg: Any) -> None: ...
    def setFromXML(
        self,
        columnNames: List[String],
        columnTypes: List[Class],
        encodedData: String,
        rowCount: int,
    ) -> None: ...
    def setValueAt(self, row: int, col: int, value: Any) -> None: ...

class JsonPath(Object):
    ROOT: JsonPath
    def createChildPath(self, arg: Union[JsonPath, int, String]) -> JsonPath: ...
    def getAsLinkedList(self) -> Any: ...
    def getDepth(self) -> int: ...
    def getKey(self) -> Object: ...
    def getParent(self) -> JsonPath: ...
    def getPathElements(self) -> List[JsonPath]: ...
    def getSubPath(self, startingDepth: int) -> JsonPath: ...
    def isAncestorOf(self, element: JsonPath) -> bool: ...
    def isRelatedTo(self, other: JsonPath) -> bool: ...
    def isRoot(self) -> bool: ...
    @staticmethod
    def isValidIdentifier(test: String) -> bool: ...
    @staticmethod
    def parse(path: String) -> String: ...

class Path:
    SERIALIZATION_WHITELIST: Set[Class]
    def getLastPathComponent(self) -> String: ...
    def getParentPath(self) -> Path: ...
    def getPathComponent(self, i: int) -> String: ...
    def getPathLength(self) -> int: ...
    def isAncestorOf(self, path: Path) -> bool: ...

class QualifiedPath(Object):
    def extend(self, id_, value) -> None: ...
    def getFirstPathComponent(self) -> None: ...
    def getFirstPathComponentId(self) -> None: ...

class TypeUtilities(Object):
    DATE_FORMAT_STRING: String
    NULL_SAFE_CASE_INSENSITIVE_ORDER: Comparator
    @staticmethod
    def anyEqual(value: T, *args: T) -> bool: ...
    @staticmethod
    def coerce(value: Object, destType: Class) -> Object: ...
    @staticmethod
    def coerceForLocale(
        value: Object, target: Class, valueLocale: Locale
    ) -> Object: ...
    @staticmethod
    def coerceGeneric(value: Object, destType: Class) -> T: ...
    @staticmethod
    def coerceLocaleSafe(str: String, type: Class) -> Object: ...
    @staticmethod
    def coerceNullSafe(value: Object, destType: Class) -> Object: ...
    @staticmethod
    def coerceNumberForLocale(
        value: Object, destType: Class, locale: Locale
    ) -> Object: ...
    @staticmethod
    def coerceNumberNullSafe(
        value: Object, destType: Class, locale: Locale
    ) -> Object: ...
    @staticmethod
    def colorToHex(c: Color) -> String: ...
    @staticmethod
    def compareInts(foo: int, bar: int) -> int: ...
    @staticmethod
    def compareNullHigh(c1: T, c2: T) -> T: ...
    @staticmethod
    def compareNullLow(c1: T, c2: T) -> T: ...
    @staticmethod
    def datasetFromJSON(json: JSONObject) -> Dataset: ...
    @staticmethod
    def datasetFromJsonString(jsonStr: String) -> Dataset: ...
    @staticmethod
    def datasetToGson(data: Dataset, gson: Optional[Gson] = ...) -> JsonElement: ...
    @staticmethod
    def datasetToJSON(data: Dataset) -> JSONObject: ...
    @staticmethod
    def deepEquals(o1: Object, o2: Object, checkArrayTypes: bool) -> bool: ...
    @staticmethod
    def equalsIgnoreCase(o1: Object, o2: Object) -> bool: ...
    @staticmethod
    def fromString(value: String, dest: Class, locale: Locale) -> Object: ...
    @staticmethod
    def getColorFromString(color: String) -> Color: ...
    @staticmethod
    def getFirstOrNull(list_: List[T]) -> T: ...
    @staticmethod
    def getInitValueForClass(c: Class) -> Object: ...
    @staticmethod
    def getLastNameComponent(name: String) -> String: ...
    @staticmethod
    def getPrimitiveType(c: Class) -> Object: ...
    @staticmethod
    def getWrapperType(c: Class) -> Class: ...
    @staticmethod
    def gsonToPy(element: JsonElement) -> PyObject: ...
    @staticmethod
    def hasPrimitiveType(c: Class) -> bool: ...
    @staticmethod
    def hasValueChanged(
        currentValue: QualifiedValue,
        previousValue: QualifiedValue,
        expectedType: DataType,
        deadband: float,
    ) -> bool: ...
    @staticmethod
    def isAssignable(dest: Class, source: Class) -> bool: ...
    @staticmethod
    def isBoolean(clazz: Class) -> bool: ...
    @staticmethod
    def isDirectlyAssignable(dest: Class, source: Class) -> bool: ...
    @staticmethod
    def isFractional(clazz: Class) -> bool: ...
    @staticmethod
    def isNullOrEmpty(s: String) -> bool: ...
    @staticmethod
    def isNumber(clazz: Class) -> bool: ...
    @staticmethod
    def isPrimitive(clazz: Class) -> bool: ...
    @staticmethod
    def isProperNumber(clazz: Class) -> bool: ...
    @staticmethod
    def neq(o1: Object, o2: Object) -> bool: ...
    @staticmethod
    def pyToGson(
        pyObject: PyObject, customGson: Optional[Gson] = ...
    ) -> JsonElement: ...
    @staticmethod
    def pyToJava(pyObject: PyObject) -> Object: ...
    @staticmethod
    def setArrayValue(arrayValue: Object, newVal: Object, pos: int) -> QualityCode: ...
    @staticmethod
    def setClassInitializer(init: TypeUtilities.ClassInitializer) -> None: ...
    @staticmethod
    def toBool(value: Object) -> bool: ...
    @staticmethod
    def toByteArray(uuid: UUID) -> bytearray: ...
    @staticmethod
    def toColor(color: Object) -> Color: ...
    @staticmethod
    def toDataset(value: Object) -> Dataset: ...
    @staticmethod
    def toDate(value: Object) -> Date: ...
    @staticmethod
    def toDocument(value: Object) -> DocumentElement: ...
    @staticmethod
    def toDouble(value: Object) -> float: ...
    @staticmethod
    def toEnum(enumType: Class, value: String) -> T: ...
    @staticmethod
    def toFloat(value: Object) -> float: ...
    @staticmethod
    def toInteger(value: Object) -> int: ...
    @staticmethod
    def toLong(value: Object) -> long: ...
    @staticmethod
    def toNumber(value: Object, locale: Optional[Locale] = ...) -> Number: ...
    @staticmethod
    def toShort(value: Object) -> int: ...
    @staticmethod
    def toStr(value: Object) -> String: ...
    @staticmethod
    def toStringLocalized(value: Object, locale: Optional[Locale] = ...) -> String: ...
    @staticmethod
    def toStringOnlyNumberLocalized(value: Object, locale: Locale) -> String: ...
    @staticmethod
    def toUUID(barr: bytearray) -> UUID: ...

    class ClassInitializer:
        def createNew(self, claz: Class) -> Object: ...



from typing import List

from java.lang import String

def loadFromFile(filePath: String, projectName: String, mode: int) -> None: ...
def removeGroups(projectName: String, paths: List[String]) -> None: ...



from typing import Any

class AlarmQueryResult:
    def getAssociatedDate(self, uuid) -> None: ...
    def getDataSet(self) -> None: ...
    def getEvent(self, uuid) -> None: ...

class AlarmQueryResultImpl(AlarmQueryResult):
    def __init__(self, *args: Any) -> None: ...
    def __iter__(self): ...
    def getAssociatedDate(self, uuid) -> None: ...
    def getDataSet(self) -> None: ...
    def getEvent(self, uuid) -> None: ...



from java.lang import Object

class PyChartScope(Object):
    def __set__(self, instance, value) -> None: ...
    def __setattr__(self, key, value) -> None: ...
    def __setitem__(self, key, value) -> None: ...



from typing import Any, List, Optional, Union

from java.lang import CharSequence as CharSequence
from java.lang import Object as Object
from java.lang import String as String
from java.lang import StringBuffer as StringBuffer
from java.lang import StringBuilder as StringBuilder
from java.util.function import Function as Function
from java.util.function import Predicate as Predicate

class MatchResult:
    def end(self, group: Optional[int] = ...) -> int: ...
    def group(self, group: Optional[int] = ...) -> String: ...
    def groupCount(self) -> int: ...
    def start(self, group: Optional[int] = ...) -> int: ...

class Matcher(Object, MatchResult):
    def appendReplacement(
        self, sb: Union[StringBuffer, StringBuilder], replacement: String
    ) -> Matcher: ...
    def appendTail(
        self, sb: Union[StringBuffer, StringBuilder]
    ) -> Union[StringBuffer, StringBuilder]: ...
    def end(self, group: Optional[int] = ...) -> int: ...
    def find(self, start: Optional[int] = ...) -> bool: ...
    def group(self, group: Optional[Union[int, String]] = ...) -> String: ...
    def groupCount(self) -> int: ...
    def hasAnchoringBounds(self) -> bool: ...
    def hasTransparentBounds(self) -> bool: ...
    def hitEnd(self) -> bool: ...
    def lookingAt(self) -> bool: ...
    def matches(self) -> bool: ...
    def pattern(self) -> Pattern: ...
    @staticmethod
    def quoteReplacement(s: String) -> String: ...
    def region(self, start: int, end: int) -> Matcher: ...
    def regionEnd(self) -> int: ...
    def regionStart(self) -> int: ...
    def replaceAll(self, arg: Union[Function, String]) -> String: ...
    def replaceFirst(self, arg: Union[Function, String]) -> String: ...
    def requireEnd(self) -> bool: ...
    def reset(self, input: Optional[CharSequence] = ...) -> Matcher: ...
    def results(self) -> Any: ...
    def start(self, group: Optional[int] = ...) -> int: ...
    def toMatchResult(self) -> MatchResult: ...
    def useAnchoringBounds(self, b: bool) -> Matcher: ...
    def usePattern(self, newPattern: Pattern) -> Matcher: ...
    def useTransparentBounds(self, b: bool) -> Matcher: ...

class Pattern(Object):
    CANON_EQ: int
    CASE_INSENSITIVE: int
    COMMENTS: int
    DOTALL: int
    LITERAL: int
    MULTILINE: int
    UNICODE_CASE: int
    UNICODE_CHARACTER_CLASS: int
    UNIX_LINES: int
    def asMatchPredicate(self) -> Predicate: ...
    def asPredicate(self) -> Predicate: ...
    @staticmethod
    def compile(regex: String, flags: Optional[int] = ...) -> Pattern: ...
    def flags(self) -> int: ...
    def matcher(self, input: CharSequence) -> Matcher: ...
    @staticmethod
    def matches(regex: String, input: CharSequence) -> bool: ...
    def pattern(self) -> String: ...
    @staticmethod
    def quote(s: String) -> String: ...
    def split(
        self, input: CharSequence, limit: Optional[int] = ...
    ) -> List[String]: ...
    def splitAsStream(self, input: CharSequence) -> Any: ...



from typing import Optional, Tuple

from java.lang import String
from java.util import EventObject

def getRoles() -> Tuple[String, ...]: ...
def getUserRoles(
    username: String, password: String, authProfile: String = ..., timeout: int = ...
) -> Optional[Tuple[String, ...]]: ...
def getUsername() -> String: ...
def isScreenLocked() -> bool: ...
def lockScreen(obscure: Optional[bool] = ...) -> None: ...
def logout() -> None: ...
def switchUser(
    username: String,
    password: String,
    event: EventObject,
    hideError: Optional[bool] = ...,
) -> bool: ...
def unlockScreen() -> None: ...
def validateUser(
    username: String,
    password: String,
    authProfile: Optional[String] = ...,
    timeout: Optional[int] = ...,
) -> bool: ...



from typing import Any, Dict, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.sfc.api import PyChartScope
from java.lang import String

def cancelChart(instanceId: String) -> None: ...
def getRunningCharts(chartPath: Optional[String] = ...) -> BasicDataset: ...
def getVariables(instanceId: String) -> PyChartScope: ...
def pauseChart(instanceId: String) -> None: ...
def redundantCheckpoint(instanceId: String) -> None: ...
def resumeChart(instanceId: String) -> None: ...
def setVariable(
    instanceId: String, stepId: String, variableName: String, variableValue: Any
) -> None: ...
def setVariables(
    instanceId: String, stepId: String, variableMap: Dict[String, Any]
) -> None: ...
def startChart(
    projectName: String, chartPath: String, arguments: Dict[String, Any]
) -> String: ...
